/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <string.h>
#include "pme.h"
#include "ucsd_consts.h"
#include "ucsd_syscom.h"
#include "ucsd_dir.h"
#include "memory.h"
#include "unit_io.h"
#include "boot.h"
#include "interpreter.h"

#define	SYSCOM		2
#define UCSD_BOOT_VOL	4

extern int dflag;
extern mem_t memory[];
extern struct int_seg_entry int_seg_tbl[];

extern struct syscom_rec *syscom;
extern word sp;
extern word np;
extern word ipc;

/* ------------------------------------------------------------------------- */
void init_syscom()
{
memset( syscom, '\0', sizeof( struct syscom_rec ) );

syscom->sysunit			= UCSD_BOOT_VOL;
syscom->gdirp  			= NIL;
syscom->memtop			= MEM_SIZE & 0xfff;
syscom->crtctl.home		= 0x0D;	/* [EM] - home */
syscom->crtctl.ndfs		= 0x21;	/* [FS] - non-destructive forward space */
syscom->crtctl.bs		= 0x08;	/* [BS] - backspace */
syscom->crtctl.fc		= 0x05;	/* fillcount */
syscom->crtctl.clearline	= 0x05;	/* clear line */
syscom->crtctl.clearscreen	= 0x05;	/* clear screen */
syscom->crtinfo.height		= 0x18;	/* height */
syscom->crtinfo.width		= 0x50;	/* width */
syscom->crtinfo.up		= 0x1F;	/* [US] - up */
syscom->crtinfo.down		= 0x0A;	/* [LF] - down */
syscom->crtinfo.left		= 0x08;	/* [BS] - left */
syscom->crtinfo.right		= 0x1C;	/* [FS] - right */
syscom->crtinfo.syeof		= 0x03;	/* ^C */
syscom->crtinfo.flush		= 0x06;	/* ^F */
syscom->crtinfo.stop		= 0x13;	/* ^S */
syscom->crtinfo.chardel		= 0x08;	/* ^H - chardel */
syscom->crtinfo.badch		= 0x3F;	/* ? - badch */
syscom->crtinfo.linedel		= 0x7F;	/* [del] - linedel */
syscom->crtinfo.altmode		= 0x1B;	/* [esc] - altmode */
}

/* ------------------------------------------------------------------------- */
int boot_system_pascal()
{
int io_result;
int index;
int num_entries;
struct ucsd_dir_entry *directory;
int sys_pas_dfirstblk;
int sys_pas_sysblk;
int sys_pas_dlastblk;
struct disk_seg_entry *rel_seg_tbl;

memset( memory, 0x00, MEM_SIZE );

/* init system comm area */
syscom = (struct syscom_rec *) &(memory[ SYSCOM ]);
init_syscom();
np = sizeof( struct syscom_rec ) + SYSCOM;
np = 256;
syscom->stkbase = syscom->memtop;

/* get the root directory from the disk */
io_result = unit_read( syscom->sysunit, np, UCSD_DIR_SIZE, UCSD_DIRBLK, 0 );
if ( io_result != 0 )
	return io_result;

/* find SYSTEM.PASCAL */
directory = (struct ucsd_dir_entry *) &(memory[ np ]);
num_entries = directory->d.v_ent.dnumfiles;

#ifdef _DEBUG_H_
if ( DEBUG2 )
	list_directory( directory );
#endif

index = 1;
while ( ( index <= num_entries ) &&
	( strncasecmp( &(directory[ index ].d.t_ent.dtid[ 1 ]),
				"SYSTEM.PASCAL",
				directory[ index ].d.t_ent.dtid[ 0 ] & 0xff ) != 0 )
		)
		index++;
if ( index > num_entries )
	{
	fprintf( stderr, "Unable to find SYSTEM.PASCAL on #%d\n", syscom->sysunit );
	return 1;
	}

#ifdef _DEBUG_H_
if ( DEBUG2 )
	list_tid_entry( &(directory[ index ]) );
#endif

sys_pas_dfirstblk = directory[ index ].dfirstblk;
sys_pas_sysblk = directory[ index ].dfirstblk + 1;
sys_pas_dlastblk = directory[ index ].dlastblk;

/* load seg table */
io_result = unit_read( syscom->sysunit, np, 0x40, sys_pas_dfirstblk, 0 );
if ( io_result != 0 )
	return io_result;

/* build working seg table */
rel_seg_tbl = (struct disk_seg_entry *) &(memory[ np ]);
for ( index = 0; index <= MAXSEG; index++ )
	{
	syscom->segtbl[ index ].code_unit = syscom->sysunit;
	syscom->segtbl[ index ].disk_addr = rel_seg_tbl[ index ].disk_addr + sys_pas_dfirstblk;
	syscom->segtbl[ index ].code_len = rel_seg_tbl[ index ].code_len;
	}

/* read seg[ 0 ] into highest memory available */
syscom->stkbase = syscom->stkbase - syscom->segtbl[ 0 ].code_len;
io_result = unit_read( syscom->sysunit,
			syscom->stkbase,
			syscom->segtbl[ 0 ].code_len,
			syscom->segtbl[ 0 ].disk_addr,
			0
			);
if ( io_result != 0 )
	return io_result;

for ( index = 0; index <= MAXSEG; index++ )
	{
	int_seg_tbl[ index ].ref_count = 0;
	int_seg_tbl[ index ].code_addr = 1;
	}
int_seg_tbl[ 0 ].ref_count = 1;
int_seg_tbl[ 0 ].code_addr = syscom->stkbase;

/* init p-m/c registers */
sp           = syscom->stkbase;
syscom->seg  = syscom->memtop - 2;
syscom->jtab = ( syscom->seg - 2 ) - mem_rd_w( syscom->seg - 2 );
ipc          = ( syscom->seg - 4 ) - mem_rd_w( syscom->seg - 4 );
sp           = sp - mem_rd_w( syscom->jtab - 8 );

/* build first MSCW */
push_w( SYSCOM );
push_w( sp );		/* localdata	dummy state save */
push_w( sp - 4 );	/* msipc	address of abort opcode */
push_w( 0xd6 );		/* msseg	ABORT opcode */
push_w( 0xd6 );		/* msjtab	ABORT opcode */
push_w( sp - 4 );	/* dyn		self referencing */
push_w( sp - 2 );	/* stat		self referencing */

syscom->lastmp = sp;
syscom->stkbase = sp;

return io_result;
}
