/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include "pme.h"
#include "ucsd_consts.h"
#include "unit_io.h"

extern int dflag;

extern struct unit_entry unit_table[ MAX_UNITS ];

#define SYM_NONE	0
#define SYM_EOF		1
#define SYM_EOL		2
#define SYM_IDENT	3
#define SYM_NUMBER	4
#define SYM_EQUALS	5
#define SYM_COLON	6
#define SYM_ESC		7
#define SYM_HASH	8

#define STATE_NONE	0
#define STATE_ID	1
#define STATE_PARAM	2

#define STR_LEN		80

int token;
char symbol[ STR_LEN ];

#define PRM_VOL		0
#define PRM_FD		1
#define PRM_TYPE	2
#define PRM_FILE	3
#define PRM_FLAGS	4

struct params
	{
	char *name;
	int id;
	} ;

struct params param_table[] =
	{
	{ "vol",      PRM_VOL },
	{ "fd",       PRM_FD },
	{ "type",     PRM_TYPE },
	{ "filename", PRM_FILE },
	{ "flags",    PRM_FLAGS },
	{ NULL,       -1 },
	} ;

struct types
	{
	char *name;
	int id;
	} ;

struct types type_table[] =
	{
	{ "console", UNIT_CONSOLE },
	{ "disk",    UNIT_DISK },
	{ "graphic", UNIT_GRAPHIC },
	{ "null",    UNIT_NULL },
	{ "printer", UNIT_PARALLEL },
	{ "remote",  UNIT_SERIAL },
	{ NULL,      -1 },
	} ;

struct flags
	{
	char *name;
	int id;
	} ;

struct flags flags_table[] =
	{
	{ "O_RDONLY", O_RDONLY},
	{ "O_RDWR",   O_RDWR },
	{ NULL,       -1 },
	} ;

int volume;

void set_var( char *param, char *value )
{
int index;
int prm_id;

index = 0;
while ( ( param_table[ index ].name != NULL ) && (strcasecmp( param_table[ index ].name, param ) != 0 ) )
	index++;

prm_id = param_table[ index ].id;
switch ( prm_id )
	{
	case PRM_VOL :
		sscanf( value, "%d", &volume );
		break;
	case PRM_FD :
		sscanf( value, "%d", &(unit_table[ volume ].fd) );
		break;
	case PRM_TYPE :
		index = 0;
		while ( ( type_table[ index ].name != NULL ) && (strcasecmp( type_table[ index ].name, value ) != 0 ) )
			index++;
		unit_table[ volume ].type = type_table[ index ].id;
		break;
	case PRM_FILE :
		unit_table[ volume ].device = (char *) malloc( strlen( value ) + 1 );
		strncpy( unit_table[ volume ].device, value, strlen( value ) );
		break;
	case PRM_FLAGS :
		unit_table[ volume ].flags = 0;
		index = 0;
		while ( flags_table[ index ].name != NULL )
			{
			if ( strcasecmp( flags_table[ index ].name, value ) == 0 )
				unit_table[ volume ].flags |= flags_table[ index ].id;
			index++;
			}
		break;
	default :
		break;
	}
}

void default_config()
{
unit_table[  1 ].type = UNIT_CONSOLE;
unit_table[  1 ].fd = 1;

unit_table[  2 ].type = UNIT_CONSOLE;
unit_table[  2 ].fd = 0;

unit_table[  4 ].type = UNIT_DISK;
unit_table[  4 ].device = "i.5";
unit_table[  4 ].flags = O_RDONLY;
}

void read_token( FILE *cf )
{
char ch;
int have_token;
int in_comment;
int symbol_ptr;

symbol_ptr = 0;
memset( symbol, '\0', sizeof( symbol ) ) ;
token = SYM_NONE;
have_token = 0;
in_comment = 0;
do
	{
	ch = fgetc( cf );
	if ( in_comment )
		switch ( ch )
			{
			case EOF :
				token = SYM_EOF;
				have_token = 1;
				break;
			case 0x0a :
				in_comment = 0;
				break;
			case '\\' :
				ch = fgetc( cf ); /* whatever the next char is we want to eat it */
				break;
			default :
				break;
			}
	else
		switch ( ch )
			{
			case EOF :
				token = SYM_EOF;
				have_token = 1;
				break;
			case '#' :
				in_comment = 1;
				break;
			case 0x09 :
				break;
			case ' ' :
				break;
			case '\\' :
				ch = fgetc( cf );
				if ( ch != 0x0a )
					; /* we want the next char as a literal */
				break;
			case '=' :
				if ( token == SYM_NONE )
					token = SYM_EQUALS;
				else
					if ( ungetc( ch, cf ) != ch )
						;
				have_token = 1;
				break;
			case ':' :
				if ( token == SYM_NONE )
					token = SYM_COLON;
				else
					if ( ungetc( ch, cf ) != ch )
						;
				have_token = 1;
				break;
			case 0x0a :
				if ( token == SYM_NONE )
					token = SYM_EOL;
				else
					if ( ungetc( ch, cf ) != ch )
						;
				have_token = 1;
				break;
			default :
				if ( isprint( ch ) )
					{
					if ( token == SYM_NONE )
						token = SYM_IDENT;
					if ( token == SYM_IDENT )
						{
						if ( symbol_ptr < (sizeof( symbol ) - 1) )
							{
							symbol[ symbol_ptr ] = ch;
							symbol_ptr++;
							symbol[ symbol_ptr ] = '\0';
							}
						}
					else
						{
						if ( ungetc( ch, cf ) != ch )
							;
						have_token = 1;
						}
					}
				else
					{
					printf( "Syntax error at character: %02x\n", ch );
					have_token = 1;
					}
				break;
			}
	} while ( ! have_token );
}

void read_config( char *config_file )
{
FILE *cf;
int state;
int rhs;
char type[ STR_LEN ];
char value[ STR_LEN ];
char ident[ STR_LEN ];


/* make sure we have something that might work */
default_config();

if ( config_file == NULL )
	return;
cf = fopen( config_file, "r" );
if ( cf == NULL )
	return;

rhs = 0;
state = STATE_ID;
do
	{
	read_token( cf );
	switch ( state )
		{
		case STATE_NONE :
			if ( token == SYM_EOL )
				state = STATE_ID;
			rhs = 0;
			break;
		case STATE_ID :
			switch ( token )
				{
				case SYM_IDENT :
					strncpy( type, "vol", sizeof( type ) );
					strncpy( ident, symbol, sizeof( ident ) );
					state = STATE_PARAM;
					break;
				case SYM_EOL :
					break;
				default :
					state = STATE_NONE;
					break;
				}
			break;
		case STATE_PARAM :
			switch ( token )
				{
				case SYM_EOL :
					state = STATE_ID;
					break;
				case SYM_IDENT :
					strncpy( ident, symbol, sizeof( ident ) );
					break;
				case SYM_EQUALS :
					strncpy( type, ident, sizeof( type ) );
					rhs = 1;
					break;
				case SYM_COLON :
					strncpy( value, ident, sizeof( value ) );
					set_var( type, value );
					rhs = 0;
					break;
				default :
					break;
				}
			break;
		default :
			break;
		}
	} while ( token != SYM_EOF );

fclose( cf );
}
