/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <curses.h>
#include "pme.h"
#include "ucsd_consts.h"
#include "ucsd_syscom.h"
#include "unit_io.h"
#include "memory.h"
#include "interpreter.h"
#include "console.h"

#define DEBUG_CONSOLE	0

extern int dflag;

extern struct unit_entry unit_table[];
extern struct syscom_rec *syscom;
extern mem_t memory[];

WINDOW *console;

int kbd_esc_seq;
int kbd_esc_sq_seq;
int kbd_esc_sq_num_seq;
char kbd_esc_sq_num;

int flush_flag;
int scr_esc_seq;
int scr_eq_seq;
int got_y;
int new_y;
int new_x;
int y;
int x;
int dle_expand;

/* ------------------------------------------------------------------------- */
word unit_read_console( word unit_num, word buffer_addr, word length, word block )
{
word index;
byte ch;

if ( DEBUG0 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_read_console: %d, %d, %d, %d\n", unit_num, buffer_addr, length, block );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

if ( unit_num == 1 )
	echo();
index = 0;
while ( index < length )
	{
	ch = getch() & 0xff;
	if ( kbd_esc_seq )
		{
		if ( kbd_esc_sq_seq )
			{
			if ( kbd_esc_sq_num_seq )
				{
				switch ( ch )
					{
					case '~' :
						switch ( kbd_esc_sq_num )
							{
							case '2' : ch = 'i'; break;
							case '3' : ch = syscom->crtinfo.chardel; break;
							case '5' : ch = 'o'; break;
							case '6' : ch = 'p'; break;
							default  : ch = 0; break;
							}
						break;
					default  : ch = 0; break;
					}
				kbd_esc_seq = 0;
				kbd_esc_sq_seq = 0;
				kbd_esc_sq_num_seq = 0;
				}
			else
				if ( isdigit( ch ) )
					{
					kbd_esc_sq_num = ch; 
					kbd_esc_sq_num_seq = 1; 
					ch = 0;
					}
				else
					{
					switch ( ch )
						{
						case 'A' : ch = syscom->crtinfo.up; break;
						case 'B' : ch = syscom->crtinfo.down; break;
						case 'C' : ch = syscom->crtinfo.right; break;
						case 'D' : ch = syscom->crtinfo.left; break;
						/*case 'H' : ch = syscom->crtinfo.home; break;*/
						default  : ch = 0; break;
						}
					kbd_esc_seq = 0;
					kbd_esc_sq_seq = 0;
					}
			}
		else
			{
			switch ( ch )
				{
				case '[' : ch = 0; kbd_esc_sq_seq = 1; break;
				case 27  : kbd_esc_seq = 0; break;
				default  : ch = 0; kbd_esc_seq = 0; break;
				}
			}
		}
	else
		{
		if ( ! isprint( ch ) )
			{
			if ( ch == syscom->crtinfo.up )			/* [US] - up */
				;
			else if ( ch == syscom->crtinfo.down )		/* [LF] - down */
				;
			else if ( ch == syscom->crtinfo.left )		/* [BS] - left */
				;
			else if ( ch == syscom->crtinfo.right )		/* [FS] - right */
				;
			else if ( ch == syscom->crtinfo.syeof )		/* ^C */
				;
			else if ( ch == syscom->crtinfo.flush )		/* ^F */
				ch = 0;
			else if ( ch == syscom->crtinfo.break_ch )	/* */
				ch = 0;
			else if ( ch == syscom->crtinfo.stop )		/* ^S */
				ch = 0;
			else if ( ch == syscom->crtinfo.chardel )	/* ^H - chardel */
				;
			else if ( ch == syscom->crtinfo.badch )		/* ? - badch */
				;
			else if ( ch == syscom->crtinfo.linedel )	/* [del] - linedel */
				;
			else if ( ch == syscom->crtinfo.altmode )	/* [esc] - altmode */
				{
				ch = 0;
				kbd_esc_seq = 1;
				}
			else if ( ch == syscom->crtinfo.etx )		/* ^C */
				;
			else if ( ch == syscom->crtinfo.backspace )	/* ^H */
				;
			else switch ( ch )
					{
					case 13 : break;
					case 27 : ch = 0; kbd_esc_seq = 1; break;
					default : ch = 0; break;
					}
			}
		}
	if ( ch != 0 )
		{
		memory[ buffer_addr + index ] = ch;
		index++;
		}
	}
if ( unit_num == 1 )
	noecho();

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_write_console( word unit_num, word buffer_addr, word length, word block )
{
word index;
char ch;

if ( DEBUG0 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_write_console: %d, %d, %d, %d\n", unit_num, buffer_addr, length, block );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

index = 0;
while ( index < length )
	{
	ch = memory[ buffer_addr + index ];
	if ( ch == syscom->crtinfo.flush )
		flush_flag = ~flush_flag;

	if ( ! flush_flag )
		{
		getyx( console, y, x );
		if ( scr_esc_seq )
			{
			if ( scr_eq_seq )
				{ /* gotoxy */
				if ( got_y )
					{
					new_x = ch - ' ';
					if ( new_x < 0 )
						new_x = 0;
					if ( x >= syscom->crtinfo.width )
						new_x = syscom->crtinfo.width - 1;
					x = new_x;
					y = new_y;
					wmove( console, y, x );
					scr_esc_seq = 0;
					scr_eq_seq = 0;
					got_y = 0;
					}
				else
					{
					new_y = ch - ' ';
					if ( new_y < 0 )
						new_y = 0;
					if ( new_y >= syscom->crtinfo.height )
						new_y = syscom->crtinfo.height - 1;
					got_y = 1;
					}
				}
			else
				switch ( ch )
					{
					case '=' :
						scr_eq_seq = 1;
						got_y = 0;
						break;
					default  :
						scr_esc_seq = 0;
						scr_eq_seq = 0;
						got_y = 0;
						break;
					}
			}
		else
			{
			if ( dle_expand )
				{
				ch -= ' ';
				while ( ch > 0 )
					{
					wprintw( console, " " );
					x++;
					ch--;
					}
				dle_expand = 0;
				}
			else if ( isprint( ch ) )
				{
				wprintw( console, "%c", ch );
				x++;
				}
			else if ( ch == syscom->crtctl.esc )		/* escape */
				scr_esc_seq = 1;
			else if ( ch == 27 )		/* escape */
				scr_esc_seq = 1;
			else if ( ch == syscom->crtctl.home )		/* [EM] - home */
				{
				x = 0;
				y = 0;
				wmove( console, y, x );
				}
			else if ( ch == syscom->crtctl.eraseeos )	/* [VT] - eraseeos */
				wclrtobot( console );
			else if ( ch == syscom->crtctl.eraseeol )	/* [GS] - eraseeol */
				wclrtoeol( console );
			else if ( ch == syscom->crtctl.ndfs )		/* [FS] - non-destructive forward space */
				{
				if ( x < syscom->crtinfo.width )
					x++;
				wmove( console, y, x );
				}
			else if ( ch == syscom->crtctl.rlf )		/* [US] - reverse line feed */
				{
				if ( y > 0 )
					y--;
				wmove( console, y, x );
				}
			else if ( ch == syscom->crtctl.bs )		/* [BS] - backspace */
				{
				if ( x > 0 )
					x--;
				wmove( console, y, x );
				}
			else if ( ch == syscom->crtctl.clearline )	/* clear line */
				{
				move( y, 0 );
				wclrtoeol( console );
				wmove( console, y, x );
				}
			else if ( ch == syscom->crtctl.clearscreen )	/* clear screen */
				{
				wclear( console );
				x = 0;
				y = 0;
				wmove( console, y, x );
				}
			else switch ( ch )
				{
				case 10 : /* [LF] cursor down */
					y++;
					if ( y >= syscom->crtinfo.height )
						{
						y = syscom->crtinfo.height - 1;
						scroll( console );
						}
					wmove( console, y, x );
					break;
				case 13 : /* [CR] */
					x = 0;
					y++;
					if ( y >= syscom->crtinfo.height )
						{
						y = syscom->crtinfo.height - 1;
						scroll( console );
						}
					wmove( console, y, x );
					break;
				case 16 : /* [DLE] expansion */
					dle_expand = 1;
					break;
				default :
					break;
				}
			}
		}
	index++;
	}
wrefresh( console );
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_clear_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_clear_console: %d\n", unit_num );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

flush_flag = 0;
scr_esc_seq = 0;
scr_eq_seq = 0;
got_y = 0;
y = 0;
x = 0;
dle_expand = 0;

kbd_esc_seq = 0;
kbd_esc_sq_seq = 0;
kbd_esc_sq_num_seq = 0;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_busy_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_busy_console: %d\n", unit_num );
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_wait_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_wait_console: %d\n", unit_num );
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_stat_console( word unit_num, word buffer_addr, word length, word dummy )
{
if ( DEBUG0 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_stat_console: %d, %d, %d, %d\n", unit_num, buffer_addr, length, dummy );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_init_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_init_console: %d\n", unit_num );

if ( unit_table[ unit_num ].device != NULL )
	{
	unit_table[ unit_num ].fd = open( unit_table[ unit_num ].device, unit_table[ unit_num ].flags );
	if ( DEBUG1 || DEBUG_CONSOLE )
		{
		if ( unit_table[ unit_num ].fd < 0 )
			fprintf( stderr, "Failed to open" );
		else
			fprintf( stderr, "Opened" );
		fprintf( stderr, " %s\n", unit_table[ unit_num ].device );
		}
	}
if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

console = initscr();
wclear( console );
raw();
noecho();
nonl();
scrollok( console, TRUE );
idlok( console, TRUE );

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_halt_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_halt_console: %d\n", unit_num );

noraw();
echo();
wrefresh( console );
endwin();

if ( ( unit_table[ unit_num ].device != NULL ) && ( unit_table[ unit_num ].fd >= 0 ) )
	{
	close( unit_table[ unit_num ].fd );
	if ( DEBUG1 || DEBUG_CONSOLE )
		fprintf( stderr, "Closed %s\n", unit_table[ unit_num ].device );
	}
return EIONOERROR;
}
