/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include "pme.h"
#include "ucsd_consts.h"
#include "unit_io.h"
#include "memory.h"
#include "disk.h"

#define DEBUG_DISK	0

extern int dflag;

extern struct unit_entry unit_table[];

extern mem_t memory[];

/* ------------------------------------------------------------------------- */
word unit_read_disk( word unit_num, word buffer_addr, word length, word block )
{
off_t offset;
off_t io_result;

offset = block * UCSD_BLKSZ;
if ( DEBUG0 || DEBUG_DISK )
	{
	fprintf( stderr, "unit_read_disk: %2u, %6u, %6u, %6u\n", unit_num, buffer_addr, length, block );
	fprintf( stderr, " file = %s, length = %6d, offset = %6ld [0x%08lx]\n", unit_table[ unit_num ].device, length, offset, offset );
	}
if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

io_result = lseek( unit_table[ unit_num ].fd, offset, SEEK_SET );
if ( io_result != offset )
	{
	fprintf( stderr, "seek failed, io_result = %ld, errno = %d\n", io_result, errno );
	perror( NULL );
	return EIODISKERR;
	}
io_result = read( unit_table[ unit_num ].fd, &(memory[ buffer_addr ]), length );
if ( io_result != length )
	{
	fprintf( stderr, "read failed, io_result = %ld, errno = %d\n", io_result, errno );
	perror( NULL );
	return EIODISKERR;
	}
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_write_disk( word unit_num, word buffer_addr, word length, word block )
{
off_t offset;
off_t io_result;

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

io_result = EIONOERROR;
offset = block * UCSD_BLKSZ;
if ( DEBUG0 || DEBUG_DISK )
	{
	fprintf( stderr, "unit_read_write: %2u, %6u, %6u, %6u\n", unit_num, buffer_addr, length, block );
	fprintf( stderr, " file = %s, length = %6d, offset = %6ld [0x%08lx]\n", unit_table[ unit_num ].device, length, offset, offset );
	}
if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

io_result = lseek( unit_table[ unit_num ].fd, offset, SEEK_SET );
if ( io_result != offset )
	{
	fprintf( stderr, "seek failed, io_result = %ld, errno = %d\n", io_result, errno );
	perror( NULL );
	return EIODISKERR;
	}
io_result = write( unit_table[ unit_num ].fd, &(memory[ buffer_addr ]), length );
if ( io_result != length )
	{
	fprintf( stderr, "write failed, io_result = %ld, errno = %d\n", io_result, errno );
	perror( NULL );
	return EIODISKERR;
	}
return EIONOERROR;
}
/* ------------------------------------------------------------------------- */
word unit_clear_disk( word unit_num )
{
if ( DEBUG1 || DEBUG_DISK )
	fprintf( stderr, "unit_clear_disk: %d\n", unit_num );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_busy_disk( word unit_num )
{
if ( DEBUG1 || DEBUG_DISK )
	fprintf( stderr, "unit_busy_disk: %d\n", unit_num );
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_wait_disk( word unit_num )
{
if ( DEBUG1 || DEBUG_DISK )
	fprintf( stderr, "unit_wait_disk: %d\n", unit_num );
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_stat_disk( word unit_num, word buffer_addr, word length, word dummy )
{
if ( DEBUG0 || DEBUG_DISK )
	fprintf( stderr, "unit_stat_disk: %d, %d, %d, %d\n", unit_num, buffer_addr, length, dummy );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_init_disk( word unit_num )
{
if ( DEBUG1 || DEBUG_DISK )
	fprintf( stderr, "unit_init_disk: %d\n", unit_num );

if ( unit_table[ unit_num ].device != NULL )
	{
	unit_table[ unit_num ].fd = open( unit_table[ unit_num ].device, unit_table[ unit_num ].flags );
	if ( DEBUG1 || DEBUG_DISK )
		{
		if ( unit_table[ unit_num ].fd < 0 )
			fprintf( stderr, "Failed to open" );
		else
			fprintf( stderr, "Opened" );
		fprintf( stderr, " %s\n", unit_table[ unit_num ].device );
		}
	}
if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_halt_disk( word unit_num )
{
if ( DEBUG1 || DEBUG_DISK )
	fprintf( stderr, "unit_halt_disk: %d\n", unit_num );
if ( ( unit_table[ unit_num ].device != NULL ) && ( unit_table[ unit_num ].fd >= 0 ) )
	{
	close( unit_table[ unit_num ].fd );
	if ( DEBUG1 || DEBUG_DISK )
		fprintf( stderr, "Closed %s\n", unit_table[ unit_num ].device );
	}
return EIONOERROR;
}
