/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include "pme.h"
#include "ucsd_consts.h"
#include "ucsd_dir.h"

#define MIN( a, b )	( (a) < (b) ? (a) : (b) )

void create_volume( int blocks, char *volume_name, char *filename )
{
int fd;
int block;
char vid[ UCSD_VIDLENG + 2 ];
unsigned char buffer[ UCSD_BLKSZ ];
struct ucsd_dir_entry directory[ 1 ];
struct tm *now_tm;
time_t now;

if ( volume_name == NULL )
	{
	fprintf( stderr, "No volume name given\n" );
	return;
	}

sprintf( vid, "%-*.*s", UCSD_VIDLENG, UCSD_VIDLENG, volume_name );
for ( block = 0; block < UCSD_VIDLENG; block++ )
	vid[ block ] = toupper( vid[ block ] );

if ( filename == NULL )
	{
	fprintf( stderr, "No file name given\n" );
	return;
	}

fd = open( filename, O_RDWR | O_CREAT, 0644 );
if ( fd < 0 )
	{
	fprintf( stderr, "Unable to open %s\n", filename );
	return;
	}

memset( buffer, 0xe5, UCSD_BLKSZ );
for ( block = 0; block < blocks; block++ )
	write( fd, buffer, UCSD_BLKSZ );

now = time( NULL );
now_tm = localtime( &now );

directory[ 0 ].dfirstblk = 0;
directory[ 0 ].dlastblk = UCSD_DIRBLK + (UCSD_DIR_SIZE / UCSD_BLKSZ);
directory[ 0 ].dfkind = UNTYPEDFILE;
memcpy( &(directory[ 0 ].d.v_ent.dvid[ 1 ]), vid, UCSD_VIDLENG );
directory[ 0 ].d.v_ent.dvid[ 0 ] = MIN( UCSD_VIDLENG, strlen( volume_name ) );
directory[ 0 ].d.v_ent.deovblk = blocks;
directory[ 0 ].d.v_ent.dnumfiles = 0;
directory[ 0 ].d.v_ent.dloadtime = 0;
directory[ 0 ].d.v_ent.dlastboot.day = now_tm->tm_mday;
directory[ 0 ].d.v_ent.dlastboot.month = now_tm->tm_mon;
directory[ 0 ].d.v_ent.dlastboot.year = now_tm->tm_year % 100;

lseek( fd, UCSD_DIRBLK * UCSD_BLKSZ, SEEK_SET );
if ( write( fd, directory, sizeof( directory ) ) != sizeof( directory ) )
	fprintf( stderr, "Failed to write UCSD directory to %s\n", filename );

close( fd );
}

void useage( char *progname )
{
fprintf( stderr, "Usage %s: [Hh?] [-b <blocks>] [-n <volume name>] filename\n", progname );
fprintf( stderr, "\t-n <volume name> set the volume name in the directory (default is 'blank')\n" );
fprintf( stderr, "\t-b <blocks>      number of blocks on the volume\n" );
fprintf( stderr, "\t-H               help, you are reading it\n" );
fprintf( stderr, "\t-h               help, you are reading it\n" );
fprintf( stderr, "\t-?               help, you are reading it\n" );
fprintf( stderr, "\n" );
}

int main( int argc, char **argv )
{
int c;
int errflag = 0;
int hflag = 0;
char *volume_name = "blank";
int blocks = 1280;

char *progname;

extern char *optarg;
extern int optind;

/* First get the program name */
progname = (char *) strrchr( argv[ 0 ], '/' );
if ( progname == NULL )
	progname = argv[ 0 ];
else
	progname++;

while( ( c = getopt( argc, argv, "b:Hh?n:" ) ) != EOF )
	switch ( c )
	{
	case( 'b' ):	blocks = atoi( optarg );
			break;
	case( 'n' ):	volume_name = optarg;
			break;
	case( 'H' ):
	case( 'h' ):
	case( '?' ):	hflag++;
			break;
	default:
			break;
	}

if ( hflag || errflag )
	{
	useage( progname );
	return ( 1 );
	}

if ( optind < argc ) /* There are additional arguments in the argument list */
	if ( strlen( argv[ optind ] ) > 0 )
		create_volume( blocks, volume_name, argv[ optind ] );

return( 0 );
}
