/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

/* unix (or DOS) format textfile to UCSD file format */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>

#define BLOCK_SIZE 1024

int convert_file( int fd, int dos )
{
int in_count;
int out_count;
int index;
char buffer[ BLOCK_SIZE ];
char cr[ 1 ];

cr[ 0 ] = 0x0d;

if ( fd < 0 )
	return 1;

/* first block is editor info */
memset( buffer, 0, BLOCK_SIZE );
out_count = write( 1, buffer, BLOCK_SIZE );

/* the rest is text */
do
	{
	in_count = read( fd, buffer, BLOCK_SIZE );
	for ( index = 0; index < in_count; index++ )
		{
		if ( isprint( buffer[ index ] ) )
			{
			write( 1, &(buffer[ index ]), 1 );
			out_count++;
			}
		else
			if  ( buffer[ index ] == 0x0a ) /* <lf> */
				{
				/* strictly when we find an <lf> we should
				look ahead for the next <lf> and decide if
				the next line will fit in the block. If not
				then we should pad the block with <nul>'s
				and start a new output block */
	
				write( 1, cr, 1 );
				out_count++;
				}
		}
	out_count = out_count % BLOCK_SIZE;
	} while ( in_count > 0 );
if ( out_count > 0 )
	{
	memset( buffer, 0, BLOCK_SIZE );
	write( 1, buffer, BLOCK_SIZE - out_count );
	}
return 0;
}

void useage( char *progname )
{
fprintf( stderr, "Usage %s: [DHh?]\n", progname );
fprintf( stderr, "          -D run as dos2ucsd\n" );
fprintf( stderr, "          -H help\n" );
fprintf( stderr, "          -h help\n" );
fprintf( stderr, "          -? help\n" );
fprintf( stderr, "\n" );
}

int main( int argc, char **argv )
{
int c;
int hflag = 0;
char *progname;

extern char *optarg;
extern int optind;

int dos = 0;

/* First get the program name */
progname = (char *) strrchr( argv[0], '/' );
if ( progname == NULL )
	progname = argv[0];
else
	progname++;

if ( strcasecmp( progname, "dos2ucsd" ) == 0 )
	dos++;

while((c = getopt(argc,argv,"DHh?")) != EOF)
	switch (c)
	{
	case('D'):	dos++;
			break;
	case('H'):
	case('h'):
	case('?'):	hflag++;
			break;
	default:
			break;
	}

if (hflag )
	{
	useage( progname );
	return 1;
	}

/* where are the additional arguments being presented ? */
if (optind < argc) /* There are additional arguments in the argument list */
	{
	while( optind < argc )
		{
		if ( strlen( argv[ optind ] ) > 0 )
			{
			int fd;

			fd = open( argv[optind], O_RDONLY );
			convert_file( fd, dos );
			close( fd );
			optind++;
			}
		}
	}
else /* The data is on stdin */
	convert_file( 0, dos );

return 0;
}
